#!/bin/bash

# supported locales (en, ru, ?)
declare -A LOC_ENABLE=(["en"]="enabled" ["ru"]="включен")
declare -A LOC_CONN=(["en"]="Wi-Fi Connection" ["ru"]="WI-FI соединение")
declare -A LOC_FAIL=(["en"]="Wi-Fi Connection failed. Try again?" ["ru"]="Wi-Fi соединение не выполнено. Попробовать снова?")
declare -A LOC_FAIL_FINAL=(["en"]="Wi-Fi Connection failed" ["ru"]="Wi-Fi соединение не выполнено")
declare -A LOC_OK=(["en"]="Wi-Fi Connected" ["ru"]="Wi-Fi соединение выполнено")
declare -A LOC_WAIT=(["en"]="Wait for Wi-Fi connection with " ["ru"]="Ожидание Wi-Fi соединения c ")
declare -A LOC_ENTER=(["en"]="Enter the PASSWORD of the network " ["ru"]="Введите ПАРОЛЬ для выбранной сети ")
declare -A LOC_NET=(["en"]="Select network:" ["ru"]="Выберите сеть:")
declare -A LOC_COLLECT=(["en"]="Try Wi-Fi network search?" ["ru"]="Выполнить поиск Wi-Fi сетей?")
declare -A LOC_CURR=(["en"]="Current Wi-Fi connection" ["ru"]="Текущее Wi-Fi подключение")

# get current locale
CURRLOCALE=$(locale | grep 'LANG=[a-z]*' -o | sed 's/^LANG=//g')
# 'enabled' in currnet locale
ENABLED="${LOC_ENABLE["$CURRLOCALE"]}"

WIFI_CONN_TITLE="${LOC_CONN["$CURRLOCALE"]}"
WIFI_FAIL_MSG="${LOC_FAIL["$CURRLOCALE"]}"
WIFI_FAIL_FINAL_MSG="${LOC_FAIL_FINAL["$CURRLOCALE"]}"
WIFI_OK_MSG="${LOC_OK["$CURRLOCALE"]}"
WIFI_WAIT_MSG="${LOC_WAIT["$CURRLOCALE"]}"
WIFI_PWD_PROMPT="${LOC_ENTER["$CURRLOCALE"]}"
WIFI_NET_PROMPT="${LOC_NET["$CURRLOCALE"]}"
WIFI_NET_COLLECT="${LOC_COLLECT["$CURRLOCALE"]}"
WIFI_NET_CURR="${LOC_CURR["$CURRLOCALE"]}"

# get current uuid
CURRUUID=$(nmcli -f UUID,TYPE con show --active | grep wifi | awk '{print $1}')

# get wifi state
function wifistate () {
  echo "$(nmcli -fields WIFI g | sed -n 2p)"
}

# get active wifi connections
function wifiactive () {
  echo "$(nmcli con show --active | grep wifi)"
}

function if_wifistate () {
  # return a expression based on wifi state
  if [[ "$(wifistate)" =~ $ENABLED ]]; then echo "yes"; fi
  echo "no"
}

function current_connection () {
  currssid=$(iwgetid -r)
  echo $currssid
}

function nmcli_list () {

#FIELDS=SSID,SECURITY
# get list of available connections without the active connection (if it's connected)
# echo "$(nmcli --fields IN-USE,"$FIELDS" device wifi list | sed "s/^IN-USE\s//g" | sed '/*/d' | sed 's/^ *//')"
# echo "$(nmcli --fields "$FIELDS" device wifi list | sed '/^--/d')"

#  echo $(nmcli -t --fields "$FIELDS" device wifi list | sed '/^:/d' | sed s/$/';'/g)

#FIELDS=SSID,SECURITY,SIGNAL
#  nmcli -t --fields "$FIELDS" device wifi list | sed '/^:/d' > /tmp/ssid1.txt
#  cat /tmp/ssid1.txt  | sort -n -r -t ':' -k3 | cut -f1,2 -d ':' | uniq -u | tr '\n' ';' > /tmp/ssid2.txt
#  cat /tmp/ssid2.txt

FIELDS=SSID,SECURITY
   nmcli -t --fields "$FIELDS" device wifi list | sed '/^:/d' | sed 's/"/\\"/g' > /tmp/ssid1.txt

   uniq -u /tmp/ssid1.txt /tmp/ssid2.txt
   cat /tmp/ssid2.txt | tr '\n' ';' > /tmp/ssid1.txt

   cat /tmp/ssid1.txt
   rm -f /tmp/ssid*.txt
}

function kdialog_call () {
    #CURRCONNECT=$(current_connection)
    LIST=$(nmcli_list)

    IFS_SAVED="$IFS"
    IFS=';'
    FLY_DIALOG="/usr/bin/fly-dialog --title \"$WIFI_CONN_TITLE\" --icon network-wireless-100 --geometry \"400x400+50-50\" --radiolist \"$WIFI_NET_PROMPT\" "
    counter=0
    for item in ${LIST}; do
      if [ ! -z $item ];then
      FLY_DIALOG="$FLY_DIALOG $counter \"${item}\" off "
      let counter++
      fi
    done
    IFS=$IFS_SAVED

    echo "#/bin/bash" > /tmp/cmd.sh
    echo $FLY_DIALOG >> /tmp/cmd.sh
    chmod a+x /tmp/cmd.sh
    res=`/tmp/cmd.sh`
    rm -f /tmp/cmd.sh

    str=""
    if [ ! -z "$res" ]; then
     IFS_SAVED="$IFS"
     IFS=';'
     counter=0
     for item in $LIST; do
      if [ ! -z $item ];then
        if [ $counter = $res ];then
          str="$item"
          break
        fi
        let counter++
      fi
     done
     IFS=$IFS_SAVED
    fi

    echo $str
}

function get_ssid () {
#    # get fields in order
#    eval FIELDSARR=( $(cat $NMCLI_ROFI_SOURCE | awk 'BEGIN { FS=","; OFS="\n" } /^FIELDS/ \
#      { $1 = substr($1, 8); print $0; }') )
#
#    # get position of SSID field
#    for i in "${!FIELDSARR[@]}"; do
#      if [[ "${FIELDSARR[$i]}" = "SSID" ]]; then
#          SSID_POS="${i}";
#      fi
#    done
#
#    # let for arithmetical vars
#    let AWKSSIDPOS=$SSID_POS+1
#
#    # get SSID from AWKSSIDPOS
#    CHSSID=$(echo "$1" | sed  's/\s\{2,\}/\|/g' | awk -F "|" '{print $'$AWKSSIDPOS'}')
#    echo "$CHSSID"
     echo $1 | cut -d':' -f1
}

function main () {
  pkill fly-dialog

  nmcli radio wifi on

  wa=""
  CURRCONNECT=$(current_connection)

  for (( ; ; ))
  do

    if [[ "$CURRCONNECT" != '' ]]; then
      fly-dialog --title "$WIFI_CONN_TITLE" --icon network-wireless-100 --yesno "${WIFI_NET_CURR}: <b>${CURRCONNECT}</b> <br> ${WIFI_NET_COLLECT}"
    else
      fly-dialog --title "$WIFI_CONN_TITLE" --icon network-wireless-100 --yesno "${WIFI_NET_COLLECT}"
    fi
    if [ $? == "1" ]; then break; fi
    echo "wait for networks..."

    OPS=$(kdialog_call)
    if [[ $OPS == '' ]];then break; fi

    CHSSID=$(get_ssid "$OPS")
    echo selected ssid $CHSSID

    nmcli_msg=""
    if [[ "$OPS" =~ "WPA2" ]] || [[ "$OPS" =~ "WEP" ]]; then
          WIFIPASS=$(fly-dialog  --title "$WIFI_CONN_TITLE, $CHSSID" --icon network-wireless-100 --password "$WIFI_PWD_PROMPT <b>$CHSSID</b>:")
    fi

    if [[ "$CHSSID" != '' ]] && [[ "$WIFIPASS" != '' ]]; then
          fly-dialog --title "$WIFI_CONN_TITLE, $CHSSID" --icon network-wireless-100 --msgbox "$WIFI_WAIT_MSG <b>$CHSSID</b> ..." &
          #it is syncronous
          nmcli_msg=$(nmcli -w 30  dev wifi con "$CHSSID" password "$WIFIPASS")
    else
          if [[ "$CHSSID" = '' ]]; then
           break
          else
           fly-dialog --title "$WIFI_CONN_TITLE, $CHSSID" --icon network-wireless-100 --msgbox "$WIFI_WAIT_MSG <b>$CHSSID</b> ..." &
           #it is syncronous
           nmcli_msg=$(nmcli -w 30 dev wifi con "$CHSSID")
          fi
    fi

    pkill fly-dialog
    CURRCONNECT=$(current_connection)
    if [[ "$CURRCONNECT" == "$CHSSID" ]]; then
      break
    else
      fly-dialog --title "$WIFI_CONN_TITLE, $CHSSID" --icon network-wireless-100 --yesno "$nmcli_msg\n$WIFI_FAIL_MSG"
      if [[ "$?" == "1" ]]; then break; else continue; fi
    fi

  done

  if [[ "$CHSSID" != '' ]]; then
    CURRCONNECT=$(current_connection)
    if [[ "$CURRCONNECT" == "$CHSSID" ]]; then
      fly-dialog --title "$WIFI_CONN_TITLE, $CHSSID" --icon network-wireless-100 --msgbox "$WIFI_OK_MSG, $CHSSID"
      echo connected to $CHSSID
    else
      echo failed to connect to $CHSSID
    fi
  fi
}

main
